![SOURCEdefender - advanced encryption protecting your Python codebase](https://images.sourcedefender.co.uk/logo.png "SOURCEdefender - advanced encryption protecting your python codebase")

---

[![python](https://img.shields.io/pypi/pyversions/sourcedefender.svg?logo=python&logoColor=fed749&colorB=3770a0&label=)][python-url]
[![downloads](https://pepy.tech/badge/sourcedefender)][pepy-url]
[![downloads](https://pepy.tech/badge/sourcedefender/week)][pepy-url]

SOURCEdefender can protect your plaintext Python source code with AES 256-bit Encryption. There is no impact on the performance of your running application as the decryption process takes place during the import of your module or when loading your script on the command-line. Encrypted code won't run any slower once loaded from a _.pye_ file compared to loading from a _.py_ or _.pyc_ file.

As we hook directly into the import process there are no cross platform or Python version compatibility issues. Encrypted code will run on the _ANY_ target environment we support. Obviously your code needs to be written with the target platform in mind, we can't automagically make __sys.getwindowsversion()__ work on Linux.


# Features
  - Code encrypted with AES 256-bit encryption.
  - Encrypted code will run on __ANY__ supported environment.
  - Enforced expire time on encrypted code (optional).


## Supported Environments
| Architecture | Operating System | Python 3.6 | Python 3.7 | Python 3.8 | Python 3.9 |
|--            |--                |--          |--          |--          |--          |
|x86_64        |Windows           |Y           |Y           |Y           |Y           |
|x86_64        |Linux             |Y           |Y           |Y           |Y           |
|x86_64        |macOS             |Y           |Y           |Y           |Y           |
|ARMv6         |Linux             |Y           |Y           |Y           |Y           |
|ARMv7         |Linux             |Y           |Y           |Y           |Y           |


# Usage

You can distribute the sourcedefender module with any project, commercial or otherwise. However, the purchase of a license is required for the distribution of encrypted code that will be used in a commercial offering. You do not need to purchase a subscription for use in a Personal or a FOSS project. However, we would greatly appreciate that you document the usage of SOURCEdefender on your website, documentation, and in your source code. Please link back to this page where you can.


## Enterprise Subscription & Support

To create encrypted code for distribution in a commercial offering we require you to have an active subscription in place. The subscription is a rolling contract and gives you permission to create encrypted code for distribution and gain access to priority email support. Should you decide to cancel your subscription this action will revoke your permission to create encrypted code. Upon cancellation of your subscription you would be free to distribute any previously encrypted code, but you would not be able to make changes to existing files or create new ones for deploying a new feature, a bug fix, or a hot-patch. 

- £50  [Monthly Subscription][enterprise-subscription-monthly-url]
- £500 [Annual Subscription (Saving 20%)][enterprise-subscription-annual-url]

The subscription is valid for your whole organisation and not for a set person or machine.
Once you subscribe there will be a record of your purchase on our system so we can validate your usage permissions.


## How do I protect my Python source code?

Let's have a look at an example of the encryption process:

    $ cat /home/ubuntu/helloworld.py
    print("Hello World!")

This is a very basic example and produces the following output:

    $ python3 /home/ubuntu/helloworld.py
    Hello World!

We do not want anyone to get at our source code and we also don't want anyone to run this code after 1 day. When we encrypt the file we can enforce an expire time of 1 day from now with the __--ttl__ option as follows:

    $ sourcedefender encrypt --ttl=1d /home/ubuntu/helloworld.py
    SOURCEdefender v5.0.27
    
    Processing:
    
    /home/ubuntu/helloworld.py

Now the file is encrypted, its contents are as follows:

    $ cat /home/ubuntu/helloworld.pye
    -----BEGIN SOURCEDEFENDER FILE -----
    Version : 5.0.27
    
    MjB5YllqRTXjP6uuDWAWQDOwsbd+E45xKJWvoIS3MBIQxc3JfE6oQUt2PGxJQyhDCO9gOknv6ZxX
    Q+I3EnKM16W7zgYmjuWtzBAJ2n7GISKhxaCFW0qyfP2WSiFXc2xiN2N6a9NX8kOjZltDxlPktIV5
    HA00ODFKRDJPWA==
    
    ------END SOURCEDEFENDER FILE ------

Once a file has been encrypted, its extension is changed __.py__  to __.pye__, this is so the module knows its encrypted and can process it accordingly during import. All you need to remember is to inlcude __sourcedefender__ as a dependency while packaging your project and import the sourcedefender module before you attempt to use your encrypted code.


## Can I still run Python from the command-line?

Yes, you can still run scripts from the command-line, but you need to run the sourcedefender library module, and include your encrypted file name as the first argument:

    $ python3 -m sourcedefender /home/ubuntu/helloworld.pye
    Hello World!
    $

Or access the module via the usual import system:

    $ cd /home/ubuntu
    $ ls
    helloworld.pye
    $ python3
    >>>
    >>> import sourcedefender
    >>> import helloworld
    Hello World!
    >>> exit()
    $


## Integrating encrypted code with PyInstaller

PyInstaller scans your code for import statements so it knows what packages to freeze. This scanning is not possible inside encrypted code so you need to tell PyInstaller to include your encrypted files. For this example, we have the following project structure:

    pyexe.py
    lib
    ├── fibonacci.pye
    └── fib.pye


In our executable script "pyexe.py" we have the following code:

    $ cat pyexe.py
    import sourcedefender
    import fib

To ensure that PyInstaller includes our encrypted files, we need to tell it where they are with the --add-binary option. So, for the above project, we could use this command:

    $ pyinstaller --onefile --add-binary lib:. pyexe.py

If you have imports that are not in your lib folder or you are using modules installed via pip and then imported inside your encrypted codebase then the pyinstaller won't be able to detect these. To get round this, you can use the '--hidden-import' option to include them in your final build.


# Changelog

## SOURCEdefender - v5.0.27

- Remoted unrequired dependencies from requirements.txt
- Reverted back to docopt as docopt-ng was a bit [buggy][docopt-github-issue-5].
- Updated the Enterprise Subscription & Support section.

## SOURCEdefender - v5.0.26

- Updated setup.py.

## SOURCEdefender - v5.0.25

- Updated documentation (again).

## SOURCEdefender - v5.0.24

- Added Company Name and Registration/VAT Numbers to the Legal section.

## SOURCEdefender - v5.0.23

- Updated documentation

## SOURCEdefender - v5.0.22

- Disable Garbage Collection during loading of msgpack data, re-enabling it after.
- Removed compile options that were sub optimal for Windows systems.
- Added clarification to documentation of usage in Commercial projects.
- Added Enterprise Subscription url's.

## SOURCEdefender - v5.0.21

- Added a Changelog to this document :-)
- Moved away from serialized data with pickle and now use msgpack as it's much more reliable.
- Removed the '--keep' option when encrypting, this makes keeping original source files the default.
- Added a '--remove' option to delete the .py file after encrypting it.
- Replaced docopt dependency with docopt-ng.
- Stopped using atomicwrite to save files as macOS doesn't like it.


# Legal

Copyright (c) 2018-2020 SOURCEdefender. All rights reserved.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. REVERSE ENGINEERING IS STRICTLY PROHIBITED.

## Company Details

SOURCEdefender is operated by RS474 Limited, a company registered in the United Kingdom.

- Company Registration No: 12865773
- VAT Registration No: GB357943163.

The goods and services you order through this site will be purchased from RS474 Limited.

<!-- URLs -->

[python-url]: https://www.python.org
[pepy-url]: https://pepy.tech/project/sourcedefender
[pypi-url]: https://pypi.org/project/sourcedefender
[enterprise-subscription-monthly-url]: https://py.pl/1pk9p3
[enterprise-subscription-annual-url]: https://py.pl/1kETY
[sourcedefender-hello-email]: mailto:hello@sourcedefender.co.uk
[docopt-github-issue-5]: https://github.com/bazaar-projects/docopt-ng/issues/5
